RecountFu = AceLibrary("AceAddon-2.0"):new("AceConsole-2.0", "AceDB-2.0", "FuBarPlugin-2.0", "AceEvent-2.0"); 
local L = AceLibrary("AceLocale-2.2"):new("RecountFu"); 
local Tablet = AceLibrary("Tablet-2.0");
local Crayon = AceLibrary("Crayon-2.0");

L:RegisterTranslations("enUS", function() return { 
   ["tabletHint1"] = "Left-Click to toggle window visibility.", 
   ["tabletHint2"] = "Double-Left-Click to reset window position.",  
   ["tabletHint3"] = "Alt-Click to reset data.",  
   ["labelName"] = "Recount",
   ["recountFuMenuTitle"] = "Recount Options",
   ["menuOptionChooseViewTitle"] = "Choose View Mode",
   ["menuOptionChooseViewDesc"] = "Select which report to view in the Recount Window",
} end); 

L:RegisterTranslations("deDE", function() return { 
    ["tabletHint1"] = "Linksklick um das Recount-Fenster ein-/auszublenden.", 
    ["tabletHint2"] = "Doppel-Linksklick um die Fensterposition zur\195\188ckzusetzen.",      
    ["tabletHint3"] = "Alt-Click um Daten zu resetten.",  
    ["labelName"] = "Recount",
    ["recountFuMenuTitle"] = "Recount Optionen",
    ["menuOptionChooseViewTitle"] = "Ansichtsmodus w\195\164hlen",
    ["menuOptionChooseViewDesc"] = "Ausw\195\164hlen welcher Report im Recount-Fenster angezeigt werden soll.",
} end);

L:RegisterTranslations("zhTW", function() return { 
   ["tabletHint1"] = "點擊左鍵切換視窗顯示", 
   ["tabletHint2"] = "雙擊左鍵重置式窗位置",  
   ["tabletHint3"] = "Alt-點擊重置資料",  
   ["labelName"] = "Recount",
   ["recountFuMenuTitle"] = "Recount 選項",
   ["menuOptionChooseViewTitle"] = "選擇觀看模式",
   ["menuOptionChooseViewDesc"] = "選擇何種報告顯示於 Recount 視窗",
} end); 

RecountFu.hasIcon = true; 
RecountFu.defaultPosition = "RIGHT"; 
RecountFu:RegisterDB("RecountFuDB"); 

RecountFu:RegisterDefaults("profile", {
    text = {
        dps_type = "Last Fight",
        show_dps = false,
        update_time = 0.5,
        show_combat_time = false
    }
})

function RecountFu:OnTextUpdate()
  self:DoTextUpdate()
end

function RecountFu:OnTooltipUpdate() 
    cat = Tablet:AddCategory(
		'columns', 3,
        'text',"Session"
	)
    self:GetTooltipSection(cat,"OverallData")
    cat = Tablet:AddCategory(
		'columns', 3,
        'text',"Last Fight"
	)
    self:GetTooltipSection(cat,"Fight1")
   Tablet:SetHint(L["tabletHint1"] .. "\n" .. L["tabletHint2"].. "\n" .. L["tabletHint3"]);
end 


function RecountFu:GetTooltipSection(cat,tableToUse)
    cat:AddLine({text = "", text2="Total", text3="Per Second"})
    cat:AddLine(self:GetTooltipLine("Damage Done",tableToUse,"Damage"))
    cat:AddLine(self:GetTooltipLine("Healing Done",tableToUse,"Healing"))
    cat:AddLine(self:GetTooltipLine("Damage Taken",tableToUse,"DamageTaken"))
    cat:AddLine(self:GetTooltipLine("Healing Taken",tableToUse,"HealingTaken"))
    cat:AddLine(self:GetDamageHealingSummaryTooltipLine(tableToUse))
end

function RecountFu:GetTooltipLine(title,tableToUse,valueToUse)
    line = {}
    line.text = title
    line.text2 = self:getValue(tableToUse,valueToUse)
    line.text3 = self:getValuePerSecond(tableToUse,valueToUse)
    return line
end

function RecountFu:GetDamageHealingSummaryTooltipLine(tableToUse)
    local sumDmgHeal = self:getValue(tableToUse,"HealingTaken")-self:getValue(tableToUse,"DamageTaken")
    if sumDmgHeal == 0 then
        line.text = "No Damage Change"
        line.text2 = ""
    elseif sumDmgHeal >0 then
        line.text = "Heal Summary"
        line.text2 = sumDmgHeal
    else
        line.text = "Damage Summary"
        line.text2 = sumDmgHeal*-1
    end
    line.text3 = ""
    return line
end

function RecountFu:OnClick()
		if (IsAltKeyDown()) then
			Recount:ResetData();
			return
		end		
 
    if (Recount.MainWindow:IsShown()) then 
        Recount.MainWindow:Hide() 
    else 
        Recount.MainWindow:Show() 
    end    
end 

function RecountFu:OnDoubleClick()
    Recount:ResetPositions(); 
    Recount.MainWindow:Show(); 
end

function RecountFu:OnEnable()
    self:UpdateTimeChanged()
    self.LastFightDPS =0
    self.CurrentFightDPS=0
end

function RecountFu:UpdateTimeChanged()
    if(self:IsEventRegistered(self.name.."TEXT_UPDATE")) then
        self:UnregisterEvent(self.name.."TEXT_UPDATE")
    end
    if(RecountFu.db.profile.text.show_dps) then
        self:ScheduleRepeatingEvent(self.name.."TEXT_UPDATE",RecountFu.DoTextUpdate,RecountFu.db.profile.text.update_time,self)
    end
end

function RecountFu:getValuePerSecond(tableToUse,valueToUse)
    playerName = UnitName("player")
    Epsilon=0.000000000000000001
    combatData = Recount.db.char.combatants[playerName].Fights[tableToUse]
    return math.floor(10*combatData[valueToUse]/(combatData.ActiveTime+Epsilon)+0.5)/10
end

function RecountFu:getValue(tableToUse,valueToUse)
    playerName = UnitName("player")
    combatData = Recount.db.char.combatants[playerName].Fights[tableToUse]
    return combatData[valueToUse]
end

function RecountFu:getTimeInCombat(tableToUse)
    Epsilon=0.000000000000000001
    return math.floor(10*(self:getValue(tableToUse,"ActiveTime")+Epsilon))/10
end

function RecountFu:DoTextUpdate()
    local text = ""
    if(self:IsTextShown()) then
        if(RecountFu.db.profile.text.show_dps) then
            text = text ..self:GetFubarDPS()
        end
        if(RecountFu.db.profile.text.show_combat_time) then
            text = text ..self:GetFubarTimeInCombat()
        end
    else
        text = Crayon:White(L["labelName"])  
    end
    self:SetText(text)
end

function RecountFu:GetFubarDPS()
    text = Crayon:Orange(" DPS:")
    if(RecountFu.db.profile.text.dps_type == "LastFight") then
        if(Recount.InCombat) then
            self.CurrentFightDPS = self:getValuePerSecond("CurrentFightData","Damage")
            text = text .. Crayon:Red(self.CurrentFightDPS)
        else
            self.LastFightDPS = self:getValuePerSecond("Fight1","Damage")
            if(self.LastFightDPS == 0) then
                if(self.CurrentFightDPS > 0) then
                    text = text .. Crayon:Yellow(self.CurrentFightDPS)
                else
                    text = text .. Crayon:Blue("-")
                end
            else
                text = text .. Crayon:White(self.LastFightDPS)
            end
        end              
    elseif(RecountFu.db.profile.text.dps_type == "Overall") then
        text = text ..Crayon:Green(self:getValuePerSecond("OverallData","Damage"))
    end
    return text
end

function RecountFu:GetFubarTimeInCombat()
    text = Crayon:Orange(" Time:")
    if(Recount.InCombat) then
        timeInCombat=self:getTimeInCombat("CurrentFightData")
    else
        timeInCombat=self:getTimeInCombat("Fight1")
    end
    if(timeInCombat == 0) then
        text = text .. Crayon:Blue("-")
    else
        if(Recount.InCombat) then
            text = text .. Crayon:Red(timeInCombat)
        else
            text = text .. Crayon:White(timeInCombat)
        end
    end
    return text
end